/*
  Arduin0 Dice - Created June 2016
    by Don Briggs (Breadboard Electronics)

  *** This code was created for an Arduino UNO using the Arduino IDE v1.6.9 ***

  Description



  Parts list
  Arduino
  Breadboard ?





  Arduino power
  Either a 9v battery or power supply plugged into the power socket
  or a standard USB power source

  Arduino pinout
    • Pin 5v -   live to all three potentiometers (outside pin)
    • Pin GND -  ground to all three potentiometers (outside pin opposite to live)
    • Pin GND -  also to ground of servo power supply
  Potentiometers center pin
    • Pin A1 -   to center pin of potentiometer 1
    • Pin A2 -   to center pin of potentiometer 2
    • Pin A3 -   to center pin of potentiometer 3
  Servos
    • Pin 9  -   to data lead of servo 1
    • Pin 10 -   to data lead of servo 2
    • Pin 11 -   to data lead of servo 3
    Servo positive lead to positive of external power source
    Servo ground lead to ground of external power source

*/
//set up the button pins
volatile int switchPin = 2;//digital pin for display button
int selectPin = 11;//digital pin for die roll

//set up names for the LED control pins
int topLeft = 4;//top left LED, digital pin 4
int topRight = 5;//top right LED digital pin 5
int middleLeft = 6;//middle left LED digital pin 6
int middleRight = 7;//mIddle right LED from digital pin 7
int bottomLeft = 8;//bottom left LED digital pin 8
int bottomRight = 9;//bottom right LED digital pin 9
int center = 10;//center LED digital pin 10

//create and populate the 2 dimensional array for the die numbers
const int leds[6][6] {
  {7, 0},
  {3, 4, 0},
  {1, 6, 7, 0},
  {1, 2, 5, 6, 0},
  {1, 2, 5, 6, 7, 0},
  {1, 2, 3, 4, 5, 6}
};

//create and populate the 2 dimensional array for the spinner
//this is the clockwise order
const int spinnerLeds[6][1] {
  {1},
  {2},
  {4},
  {6},
  {5},
  {3}
};

volatile int buttonMenu = 0;//variable to check if button is pressed
volatile int buttonState = 0;//condition of the button, on or off
int randomRoll = 0;//the die roll random number
int loopDelay = 400;//delay used for looping through all numbers
int rollDelay = 2000;//delay used for displaying die roll

void setup() {
  pinMode(topLeft , OUTPUT);//setup the LED pins
  pinMode(topRight , OUTPUT);
  pinMode(middleLeft , OUTPUT);
  pinMode(middleRight , OUTPUT);
  pinMode(center , OUTPUT);
  pinMode(bottomLeft , OUTPUT);
  pinMode(bottomRight , OUTPUT);

  pinMode(selectPin , INPUT_PULLUP);//die roll button with pullup resistor
  pinMode(switchPin , INPUT_PULLUP);//loop button set up as interupt pin
  attachInterrupt(digitalPinToInterrupt(switchPin), SwitchButton, CHANGE);//point interupt to the SwitchButton function

  Serial.begin(9600);//turn on the serial port
  randomSeed(analogRead(0));//set up the random seed on an unconnected analog pin
  TurnOff();//start with all off
}

void loop() {
  //if (buttonMenu == 99)
  //{
  //  buttonMenu = 1;//reset and wait for another die roll
  //}

  SelectButton();//check for a die roll button press
  //if the interupt has detected a display button press, act on that here
  switch (buttonMenu)
  {
    case 1:
      {
        TurnOff();
        break;
      }
    case 2://loop through all numbers
      {
        for (int i = 0; i < 6; i++)
        {
          TurnOn(i);
        }
        break;
      }
    case 3://spin clockwise
      {
        Spinner();
        break;
      }
    case 4://spin anti clockwise
      {
        SpinnerReverse();
        break;
      }
  }



}

//random roll mode
//pressing button will cause all LEDs to flash then display number
void SelectButton()
{
  CheckButtonState();//was it pressed?
  if (buttonMenu == 99)//yes
  {
    for (int x = 0; x < 3; x++)//run the spinner 3 times
    {
      Spinner();
      SpinnerReverse();
    }
    randomRoll = random(0, 6);//get die roll
    Flash();//do the flashing

    TurnOn(randomRoll);//turn on the reauired LEDs
    delay(rollDelay);//keep the LEDs (die number) visible for a while
    TurnOff();//reset the LEDs
    buttonMenu = 1;//reset and wait for another die roll
  }
}

//loop through all numbers etc
void SwitchButton()
{
  int reading = digitalRead(switchPin);
  if (reading != buttonState) {
    buttonState = reading;
    if (buttonState == HIGH) {
      buttonMenu++;
      if (buttonMenu > 4)
      {
        buttonMenu = 1;
      }
    }
  }
}
//checks the state of the die roll button press
void  CheckButtonState()
{
  int currentReading = digitalRead(selectPin);//has the current button been pressed
  if (currentReading == 0)//has button been pressed
  {   
      buttonMenu = 99;//yes, always set this to 99    
  }
}

//turns all LEDs off
void TurnOff()
{
  digitalWrite(topLeft, LOW);
  digitalWrite(topRight, LOW);
  digitalWrite(middleLeft, LOW);
  digitalWrite(middleRight, LOW);
  digitalWrite(bottomLeft, LOW);
  digitalWrite(bottomRight, LOW);
  digitalWrite(center, LOW);
}

//turn all LEDs on
void TurnAllOn()
{
  digitalWrite(topLeft, 1);
  digitalWrite(topRight, 1);
  digitalWrite(middleLeft, 1);
  digitalWrite(middleRight, 1);
  digitalWrite(bottomLeft, 1);
  digitalWrite(bottomRight, 1);
  digitalWrite(center, 1);
}

//flash all LEDs by turning them on and off with an interval
void Flash()
{
  for (int i = 0; i <= 6; i++)//do it six times
  {
    TurnAllOn();
    delay(50);
    TurnOff();
    delay(50);
  }
}

void Spinner()
{
  for (int i = 0; i < 6; i++)//do it six times
  {
    TurnOnLed(spinnerLeds[i][0]);
    delay(60);
    // CheckButtonState(switchPin);//was it pressed?
  }
}

void SpinnerReverse()
{
  for (int i = 5; i >= 0; i--) //do it six times
  {
    TurnOnLed(spinnerLeds[i][0]);
    delay(60);
    // CheckButtonState(switchPin);//was it pressed?
  }
}

//turn on the required LEDs to display the correct number
//input parameter (dieRoll) contains the random number

//to understand the array (leds) look at the way in which it has been declared at the top of the program
//each row of the array shows which LEDs will need to be turned on for a given die roll number
//for example, if there was a dieRoll of 2, the second row of the array will be used ({3, 4, 0}),
//              so LEDs numbered 3 and 4 will need to be lit,
//              zero is the end of line marker which will break out of the loop
void TurnOn(int dieRoll)
{
  TurnOff();//start again with new die roll
  for (int i = 0; i <= 6; i++)//loop through the array
  {
    if (leds[dieRoll][i] == 0)//end of line marker so exit the loop
    {
      break;
    }
    TurnOnLed(leds[dieRoll][i]);
  }
  delay(loopDelay);
}

//turn on the individual LED according to the list in the array for that number
//if the LED is already on, turn it off or vice-versa, used for the spinners
void TurnOnLed(int whichLed)
{
  switch (whichLed)
  {
    case 1:
      {
        if (digitalRead(topLeft) == LOW)
        {
          digitalWrite(topLeft, HIGH);
        }
        else
        {
          digitalWrite(topLeft, LOW);
        }
        break;
      }
    case 2:
      {
        if (digitalRead(topRight) == LOW)
        {
          digitalWrite(topRight, HIGH);
        }
        else
        {
          digitalWrite(topRight, LOW);
        }
        break;
      }
    case 3:
      {

        if (digitalRead(middleLeft) == LOW)
        {
          digitalWrite(middleLeft, HIGH);
        }
        else
        {
          digitalWrite(middleLeft, LOW);
        }
        break;
      }
    case 4:
      {
        if (digitalRead(middleRight) == LOW)
        {
          digitalWrite(middleRight, HIGH);
        }
        else
        {
          digitalWrite(middleRight, LOW);
        }
        break;
      }
    case 5:
      {
        if (digitalRead(bottomLeft) == LOW)
        {
          digitalWrite(bottomLeft, HIGH);
        }
        else
        {
          digitalWrite(bottomLeft, LOW);
        }
        break;
      }
    case 6:
      {
        if (digitalRead(bottomRight) == LOW)
        {
          digitalWrite(bottomRight, HIGH);
        }
        else
        {
          digitalWrite(bottomRight, LOW);
        }
        break;
      }
    case 7:
      {
        if (digitalRead(center) == LOW)
        {
          digitalWrite(center, HIGH);
        }
        else
        {
          digitalWrite(center, LOW);
        }
        break;
      }
  }
  CheckButtonState();//was the die roll button pressed?
}




